unit DateTime;

Interface

uses MsgBox, Validate;

type
  PTimeValidator = ^TTimeValidator;
  TTimeValidator = object(TPXPictureValidator)
    constructor Init(AutoFill: boolean);
    procedure Error; virtual;
    function IsValid(const s:string): boolean; virtual;
    function Transfer(var S:string;Buffer:pointer;
      Flag:TVTransfer): word; virtual;
    function CheckTime(const S:string): boolean;
  end;

  TTime = array[1..2] of word;  {Time in the form : Hour:Min or Hour;Min }
{ Global time routines to be used through the whole system. }

type
  PDateValidator = ^TDateValidator;
  TDateValidator = object(TPXPictureValidator)
    constructor Init(AutoFill: boolean);
    procedure Error; virtual;
    function IsValid(const S: string): boolean; virtual;
    function Transfer(var S: string; Buffer: pointer;
      Flag: TVTransfer): word; virtual;
    function CheckDate(const S: string): boolean;
  end;

  TDate = array[1..3] of word;  { Date in the form : Month, Day, Year }

  DateModeType = (American,ANSI,British,Italian,French,German);
    { This type designates the appearance of the date for I/O usage:
           American:    'MM/DD/YYYY'
           ANSI:        'YYYY.MM.DD'
           British:     'DD/MM/YYYY'
           Italian:     'DD-MM-YYYY'
           French:      'DD/MM/YYYY'
           German:      'DD.MM.YYYY'
    }

const
  Datemode: DateModeType = American;
  DateSeparator: char = '/';
  TimeSeparator: char = ':';
      { Theses variables denotes the current mode for date I/O and
      time I/O and the separators used for this mode.
      Default it is set to American dates mm/dd/yy and time hh:mm.}

Implementation

function StringToTime(const S:string; var Hour, Min:word): boolean;
var
  p1: Word;
  code: integer;
begin
  Hour := 0; Min := 0;
  StringToTime := False;
  if S = '' then exit;  { Empty times not accepted }
  p1 := pos(';', S);
  if p1 = 0 then
  begin
    p1 := pos(':', S);
    if p1 = 0 then exit else TimeSeparator := ':';
  end {;}
  else TimeSeparator := ';';
  val(copy(S, 1, p1-1), Hour, code); if code <> 0 then exit;
  val(copy(S, p1+1, 3), Min, code); if code <> 0 then exit;
  if (Hour > 12) or (Hour < 0) or (Min > 59) or (Min < 0)
  then exit
  else StringToTime := True;
end; { StringToTime }


{ Convert a time to a string. }

procedure TimeToString(var Hour, Min:Word;var S:string);
begin
  Str(Min, S);
  S := CHR(48+Hour div 10) + CHR(48+Hour mod 10) + TimeSeparator + S;
end; { TimeToString }

constructor TTimeValidator.Init(AutoFill: boolean);
var S: string;
begin
  { Construct picture mask using the current separator for the time.
     1:30, 02:15, 12:39 etc. are all valid inputs for a time.
     The separators are put in automatically if autofill is set to true
     and 2 digits are entered for the hour and minutes }
  inherited Init('{#[#]}'+TimeSeparator+'{#[#]}', AutoFill);

  { Set transfer flag to indicate validator handles transfer }
  Options := Options or voTransfer;
end;

function TTimeValidator.IsValid(const S: string): Boolean;
begin
  IsValid := (Inherited IsValid(S)) and (Length(S) > 0) and CheckTime(S);
end;

function TTimeValidator.CheckTime(const S: string): Boolean;
var Time: TTime;
begin
  CheckTime := StringToTime(S, Time[1], Time[2]);
end;

function TTimeValidator.Transfer(var S:string;Buffer:Pointer;
  Flag:TVTransfer): Word;
var
  Time: TTime;  { Array for hour and minute values }
begin
  { Only if validator handles transfer then code is executed }
  if Options and voTransfer <> 0 then
  begin
    Transfer := SizeOf(TTime);   { Return non zero result to indicate ok }
    case Flag of
     vtGetData: { Get data from input line }
       begin
         { Extract hour and minute from string }
         StringToTime(S, Time[1], Time[2]);
         TTime(Buffer^) := Time; { Use type cast to TTime }
       end;
     vtSetData: { Set data to input line }
       begin
         { Construct string from Time }
        Time := TTime(Buffer^);     { Use type cast to TTime type }
        TimeToString(Time[1], Time[2], S);
       end;
    end;
  end
  else
    Transfer := 0;
end;

{$IFDEF Windows}
procedure TTimeValidator.Error;
var
  MsgStr: array[0..80] of Char;
begin
  wvsprintf(MsgStr, 'Value is not in the range 00:00 to 12:00.');
  MessageBox(0, MsgStr, 'Validator', mb_IconExclamation or mb_Ok);
end;
{$ELSE}
procedure TTimeValidator.Error;
begin
  MessageBox('Value not in the range 00:00 to 12:00.', nil,
    mfError + mfOKButton);
end;
{$ENDIF Windows}


 { TDate }
procedure SetDateMode(Mode: DateModeType);
begin
  DateMode := Mode;
  case DateMode of
    American, British, French: DateSeparator := '/';
    Italian:                   DateSeparator := '-';
    ANSI,German:               DateSeparator := '.';
  end;
end;

{ Convert a string representing a date to its components according to
   the current DateMode.
   If the date is not valid then the result is false and a zero date is
   returned }
function StringToDate(const S: string;var Month, Day, Year: word): boolean;
var
  p1, p2: Word;
  fc : integer;
begin
  Month := 0; Day := 0; Year := 0; fc := 0;
  StringToDate := False;
  if S='' then exit;  { Empty dates not accepted }
  p1 := pos(DateSeparator,S); if p1=0 then exit;
  p2 := pos(DateSeparator,copy(S,p1+1,255)); if p2=0 then exit;
  VAL(copy(S,1,p1-1),Day,fc); if fc<>0 then exit;
  VAL(copy(S,p1+1,p2-1),Month,fc); if fc<>0 then exit;
  VAL(copy(S,p1+p2+1,255),Year,fc); if fc<>0 then exit;
  { Correct for date mode }
  case DateMode of
    American: begin p1 := Day; Day := Month; Month := p1; end;
    ANSI:     begin p1 := Day; Day := Year; Year := p1; end;
  end;
  if Year<80 then Year := Year+1900;
  if (Day>31) OR (Month>12) OR (Day<1) OR (Month<1) OR
     ((Day>30) and (Month IN [4,6,9,11])) OR
     ((Month=2) and ((Day>29) OR (Day>28) and ((Year mod 4<>0)
      OR (Year mod 100=0) and (Year mod 400<>0))))
  then exit
  else StringToDate := True;
end; { StringToDate }

{ Convert a date to a string according to the current DateMode }
procedure DateToString(var Month, Day, Year: word; var S: string);
begin
  if (Month=0) OR (Day=0) then begin S := ''; exit; end;
  Str(Year,S);
  case DateMode of
    Ansi:    S := S+DateSeparator+CHR(48+Month div 10)+CHR(48+Month mod 10)+
                  DateSeparator+CHR(48+Day div 10)+CHR(48+Day mod 10);
    American:S := CHR(48+Month div 10)+CHR(48+Month mod 10)+DateSeparator+
                  CHR(48+Day div 10)+CHR(48+Day mod 10)+DateSeparator+S;
    else     S := CHR(48+Day div 10)+CHR(48+Day mod 10)+DateSeparator+
                  CHR(48+Month div 10)+CHR(48+Month mod 10)+DateSeparator+S;
  end;
end; { DateToString }

constructor TDateValidator.Init(AutoFill: boolean);
var S : string;
begin
  { Construct picture mask using the current separator.
     1/1/90, 1/12/90, 12/1/90, 01/01/90, 1/1/9, 1/1/1990 etc.
     are all valid inputs for for a date.
     The separators are put in automatically if autofill is set to true
     and 2 digits are entered for the month and day }
  inherited Init('{#[#]}'+DateSeparator+'{#[#]}'+DateSeparator+'{#[###]}',AutoFill);
  { Set transfer flag to indicate validator handles transfer }
  Options := Options or voTransfer;
end;

{$IFDEF Windows}
procedure TDateValidator.Error;
var
  MsgStr: array[0..80] of Char;
begin
  wvsprintf(MsgStr, ^C'Input is not a correct date.');
  MessageBox(0, MsgStr, 'Validator', mb_IconExclamation or mb_Ok);
end;
{$ELSE}
procedure TDateValidator.Error;
begin
  MessageBox(^C'Input is not a correct date.', nil, mfError + mfOKButton);
end;
{$ENDIF Windows}

function TDateValidator.IsValid(const S: string): boolean;
begin
  IsValid := (Inherited IsValid(S)) and (Length(S)>0) and CheckDate(S);
end;

function TDateValidator.CheckDate(const S: string): Boolean;
var Date : TDate;
begin
  CheckDate := StringToDate(S,Date[1],Date[2],Date[3]);
end;

function TDateValidator.Transfer(var S: string; Buffer: pointer;
  Flag: TVTransfer): word;
var
  Date : TDate;  { Array for month, day, year values }
begin
  { Only if validator handles transfer then code is executed }
  if Options and voTransfer <> 0 then
  begin
    Transfer := SizeOf(TDate);   { Return non zero result to indicate ok }
    case Flag of
     vtGetData: { Get data from input line }
       begin
         { Extract month, day and year from string }
         StringToDate(S,Date[1],Date[2],Date[3]);
         TDate(Buffer^) := Date; { Use type cast to TDate }
       end;
     vtSetData: { Set data to input line }
       begin
         { Construct string from Date }
        Date := TDate(Buffer^);     { Use type cast to TDate type }
        DateToString(Date[1],Date[2],Date[3],S);
       end;
    end;
  end
  else
    Transfer := 0;
end;

end.